local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

function CleanUI_InitDeBuffs()
    if (MASQUE) then
        buttongroup = MASQUE:Group("CleanUI", "Buffs");
    end

    local frame = CreateFrame("Frame");
    --frame:HookScript("OnUpdate", CleanUI_DeBuffsOnUpdate);
    frame:RegisterEvent("UNIT_AURA");
    
    frame:HookScript("OnEvent", CleanUI_DeBuffsOnEvent);

    CleanUI_CleanDeBuffs();
    CleanUI_StartDelay(2, CleanUI_CleanDeBuffs);

    CleanUI_DeBuffsHookEvents();
    
    CleanUI_UIParent_UpdateTopFramePositions()
end

function CleanUI_DeBuffsOnEvent(self, event, ...)
    if (event == "UNIT_AURA") then
        CleanUI_CleanDeBuffs();
    end
end

function CleanUI_DeBuffsHookEvents()
    hooksecurefunc("ActionButton_OnUpdate", CleanUI_ActionButtonDeBuffs_OnUpdate);
    hooksecurefunc("UIParent_UpdateTopFramePositions", CleanUI_UIParent_UpdateTopFramePositions);
end

function CleanUI_UIParent_UpdateTopFramePositions()
    local point, relativeTo, relativePoint, xOfs, yOfs = BuffFrame:GetPoint(1);
    BuffFrame:SetPoint(point, relativeTo, relativePoint, xOfs - 30, yOfs);
end

local function calculateAction(button)
    if (button:GetName()) then
        local _, globalID = GetActionInfo(button.action);
        local spellName;
        
        if (globalID == 216698) then
            -- Seele ernten
            spellName = GetSpellInfo(216708);
        elseif (globalID == 137378) then
            -- Hurrikan in der Flasche
            spellName = GetSpellInfo(215294);
        else
            spellName = GetSpellInfo(globalID);
        end
        
        return spellName;
    end
    
    return nil;
end

function CleanUI_CleanDeBuffs()
    if (MASQUE and buttongroup) then
        CleanUI_SkinBuffs();
    end
end

function CleanUI_SkinBuffs()
    local button;
    local debuffType, color;

    for i=1, BUFF_MAX_DISPLAY do
        button = _G["BuffButton"..i];
        if (button) then
            buttongroup:RemoveButton(button);
            buttongroup:AddButton(button);
        end
    end

    for i=1, DEBUFF_MAX_DISPLAY do
        button = _G["DebuffButton"..i];
        if (button) then
            buttongroup:RemoveButton(button);
            buttongroup:AddButton(button);
            
            -- resize debuffs (default = 30)
            button:SetWidth(36);
            button:SetHeight(36);
        end
    end
end

function CleanUI_ActionButtonDeBuffs_OnUpdate(self, elapsed)
    local spellName = calculateAction(self);
    CleanUI_DeBuffsUpdate(self, spellName);
end

function CleanUI_DeBuffsUpdate(button, spellName)
    local statusbar, shown;
    local name, icon, count, debuffType, duration, expirationTime, unitCaster, canStealOrPurge, xy, spellId, canApplyAura;
    local target, modifiedClick;

    --if (InCombatLockdown()) then
        if (spellName) then
            target = "target";
            modifiedClick = GetModifiedClick("FOCUSCAST");

            -- focus infos
            if (modifiedClick == "ALT" and IsAltKeyDown()) then
                target = "focus";
            elseif (modifiedClick == "CTRL" and IsControlKeyDown()) then
                target = "focus";
            elseif (modifiedClick == "SHIFT" and IsShiftKeyDown()) then
                target = "focus";
            end

            -- check buffs
            name, icon, count, debuffType, duration, expirationTime, unitCaster, canStealOrPurge, xy, spellId, canApplyAura = CleanUI_UnitBuff(target, spellName);

            if (name and unitCaster == "player") then
                shown = CleanUI_UpdateDeBuffInfo(button, duration, expirationTime, count);
            end

            if (not shown) then
                -- check debuffs
                name, icon, count, debuffType, duration, expirationTime, unitCaster, canStealOrPurge, xy, spellId, canApplyAura = CleanUI_UnitDebuff(target, spellName);    

                if (name and unitCaster == "player") then
                    shown = CleanUI_UpdateDeBuffInfo(button, duration, expirationTime, count);
                end
            end

            if (not shown) then
                -- self buffs
                name, icon, count, debuffType, duration, expirationTime, unitCaster, canStealOrPurge, xy, spellId, canApplyAura = CleanUI_UnitBuff("player", spellName);

                if (name and unitCaster == "player") then
                    shown = CleanUI_UpdateDeBuffInfo(button, duration, expirationTime, count);
                end
            end
        end
    --end

    if (not shown) then
        if (button) then
            local statusbar = CleanUI_GetStatusBar(button);
            statusbar:Hide();
            
            local counttext = CleanUI_GetDeBuffCount(button)
            counttext:SetText("");
        end
    end
end

function CleanUI_UpdateDeBuffInfo(button, duration, expirationTime, count)
    if (not button) then
        return nil;
    end

    local statusbar = CleanUI_GetStatusBar(button);
    statusbar:Show();



    local startTime = expirationTime - duration;

    if (expirationTime - startTime == 0) then
        statusbar:SetMinMaxValues(0, 100);
        statusbar:SetValue(100);
    else
        statusbar:SetMinMaxValues(0, expirationTime-startTime);
        statusbar:SetValue(expirationTime-GetTime());
    end
    
    local counttext = CleanUI_GetDeBuffCount(button)
    if (count > 0) then
      counttext:SetText(RED_FONT_COLOR_CODE..count..FONT_COLOR_CODE_CLOSE);
    else
      counttext:SetText("");
    end

    return 1;
end

function CleanUI_GetDeBuffCount(button)
    local name = button:GetName().."DeBuffCount";
    local counttext = _G[name];
    
    if (not counttext) then
        counttext = button:CreateFontString(name, "ARTWORK");
        counttext:SetFontObject("GameFontNormalSmall");
        counttext:SetSize(16, 16);
        counttext:SetJustifyH("LEFT");
        counttext:SetJustifyV("BOTTOM");
        counttext:ClearAllPoints();
        counttext:SetPoint("BOTTOMLEFT", 0, 3);
    end

    return counttext;
end

function CleanUI_GetStatusBar(button)
    local name = button:GetName().."DeBuffBar";
    local statusbar = _G[name];

    if (not statusbar) then
        statusbar = CreateFrame("StatusBar", name, button, "CleanUIActionButtonDeBuffBarTemplate");
    end

    local width = button:GetWidth() - 2;
    local height = 5;

    statusbar:SetSize(width, height);
    statusbar:ClearAllPoints();
    statusbar:SetPoint("BOTTOM", button, "BOTTOM", 0, 1);
    statusbar:SetStatusBarColor(1, 0, 0);

    return statusbar;
end
