CUI_MAILLOGS_MAX = 200;

local sendItems = {};

local postal = false;

-- saved variables per character
CleanUIMailLog = {};

function CleanUI_InitMail()
    CleanUI_CreateLogFrame();

    CleanUI_MailHookEvents();
    CleanUI_RegisterMailEvents();
end

function CleanUI_MailHookEvents()
    hooksecurefunc("MailFrameTab_OnClick", CleanUI_MailFrameTab_OnClick);
    hooksecurefunc("SendMailFrame_SendMail", CleanUI_SendMailFrame_SendMail);
end

function CleanUI_RegisterMailEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");
    frame:RegisterEvent("MAIL_SEND_INFO_UPDATE");
    frame:RegisterEvent("MAIL_SEND_SUCCESS");

    frame:HookScript("OnEvent", CleanUI_MailOnEvent);
end

function CleanUI_MailOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
    end
    if (event == "MAIL_SEND_INFO_UPDATE") then
        CleanUI_MailSendInfoUpdate();
    end
    if (event == "MAIL_SEND_SUCCESS") then
        sendItems = {};
    end
end

function CleanUI_MailSendInfoUpdate()
    local itemName, itemID, itemLink, itemRarity, itemTexture, stackCount, itemQuality;
    for i=1, ATTACHMENTS_MAX_SEND do
        itemName, itemID, itemTexture, stackCount, itemQuality = GetSendMailItem(i);
        if (itemName) then
            itemLink = GetSendMailItemLink(i);
            _, _, itemRarity = GetItemInfo(itemLink);
            sendItems[itemName] = {texture=itemTexture, count=stackCount, rarity=itemRarity};
        end
    end
end

function CleanUI_CreateLogFrame()
    PanelTemplates_SetNumTabs(MailFrame, 3);

    local button = CreateFrame("Button", "MailFrameTab3", MailFrame, "FriendsFrameTabTemplate");
    button:SetAttribute("id", 3);
    button:SetText(GUILD_BANK_LOG);
    button:ClearAllPoints();
    button:SetPoint("LEFT", MailFrameTab2, "RIGHT", -8, 0);
    button:HookScript("OnClick", function() MailFrameTab_OnClick(self, 3) end);

    HybridScrollFrame_OnLoad(CUIMailLogScrollFrame);
    CUIMailLogScrollFrame.update = CleanUI_UpdateMailLog;
    HybridScrollFrame_CreateButtons(CUIMailLogScrollFrame, "CUIMailLogEntryTemplate");

    local resetButton = CreateFrame("Button", "CUIMailLogResetButton", CUIMailLogScrollFrame, "UIPanelButtonTemplate");
    resetButton:SetWidth(120);
    resetButton:SetHeight(25);
    resetButton:SetPoint("TOPRIGHT", MailFrame, "TOPRIGHT", -21, -30);
    resetButton:SetText("Reset");
    resetButton:SetScript("OnClick", CleanUI_MailLogReset);
    resetButton:SetFrameLevel(resetButton:GetFrameLevel() + 1);

    local header = CUIMailLogScrollFrame:CreateFontString("CUIMailLogScrollFrameHeader");
    header:SetFontObject("GameFontNormal");
    header:SetWidth(100);
    header:SetHeight(25);
    header:SetText("Log");
end

function CleanUI_MailLogReset()
    CleanUIMailLog = {};
    CleanUI_UpdateMailLog();
end

function CleanUI_SendMailFrame_SendMail(self)

    local data = {};

    local sendTo = SendMailNameEditBox:GetText();
    local sendSubject = SendMailSubjectEditBox:GetText();
    local sendBody = SendMailBodyEditBox:GetText();
    data.to = sendTo;
    data.subject = sendSubject;
    data.body = sendBody;

    data.money = MoneyInputFrame_GetCopper(SendMailMoney);

    if ( SendMailSendMoneyButton:GetChecked() ) then
        data.moneytype = SEND_MONEY;
    else
        data.moneytype = COD;
    end

    data.items = sendItems;

    data.time = time();
    data.expires = time() + 60*60*24*30;

    tinsert(CleanUIMailLog, 1, data);
    
    while (#CleanUIMailLog > 50) do
       tremove(CleanUIMailLog, 51);
    end
end

function CleanUI_MailFrameTab_OnClick(self, tabID)
    if ( not tabID ) then
        tabID = self:GetID();
    end

    if ( tabID == 1 ) then
        InboxTitleText:Show();
        SendMailTitleText:Hide();
        CUIMailLogScrollFrameHeader:Hide();
    elseif ( tabID == 2 ) then
        InboxTitleText:Hide();
        SendMailTitleText:Show();
        CUIMailLogScrollFrameHeader:Hide();
    else    
        InboxTitleText:Hide();
        SendMailTitleText:Hide();
        CUIMailLogScrollFrameHeader:Show();
    end
    
    if ( tabID == 3 ) then
        ButtonFrameTemplate_HideButtonBar(MailFrame)
        MailFrameInset:SetPoint("TOPLEFT", 4, -58);
        InboxFrame:Hide();
        SendMailFrame:Hide();
        SetSendMailShowing(false);
        CUIMailLogScrollFrame:Show();
        CleanUI_UpdateMailLog();
    else
        CUIMailLogScrollFrame:Hide();
    end
end

function CleanUI_UpdateMailLog()
    local buttons = CUIMailLogScrollFrame.buttons;

    local numEntries = #CleanUIMailLog;
    local numButtons = #buttons;

    local scrollOffset = HybridScrollFrame_GetOffset(CUIMailLogScrollFrame);
    local buttonHeight = buttons[1]:GetHeight();
    local displayedHeight = 0;

    local entry, logIndex, data, diffExpires;
    local dayInSeconds = 24*60*60;
    local now = time();

    for i=1, numButtons do
        logIndex = i + scrollOffset;
        entry = buttons[i];
        data = CleanUIMailLog[logIndex];

        if ( logIndex <= numEntries ) then
            entry:Show();

            diffExpires = (data.expires or now) - now;

            -- weniger als ein Tag
            if (diffExpires < 0) then
                entry.time:SetText("|cffaaaaaa"..date("%m/%d/%y %H:%M:%S", data.time).."|r");
            elseif (diffExpires < dayInSeconds) then
                entry.time:SetText("|cffff0000"..date("%m/%d/%y %H:%M:%S", data.time).."|r");
            else
                entry.time:SetText("|cffffffff"..date("%m/%d/%y %H:%M:%S", data.time).."|r");
            end

            entry.subject:SetText("|cffffffff"..data.subject.."|r");
            entry.data = data;
        else
            entry:Hide();
        end

        displayedHeight = displayedHeight + buttonHeight;
    end

    HybridScrollFrame_Update(CUIMailLogScrollFrame, numEntries * buttonHeight, displayedHeight);
end

function CleanUI_MailLog_OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
    CleanUIAddons_MailLog_CreateTooltip(GameTooltip, self.data);
end

function CleanUI_MailLog_OnLeave(self)
    GameTooltip:FadeOut();
end

function CleanUIAddons_MailLog_CreateTooltip(tooltip, data)

    tooltip:SetText("|cffffffff"..data.subject.."|r");

    tooltip:AddDoubleLine(MAIL_TO_LABEL, "|cffffffff"..data.to.."|r", nil, nil, nil, nil, nil, nil);
    tooltip:AddLine("|cffaaaaaa"..date("%m/%d/%y %H:%M:%S", data.time).."|r", nil, nil, nil, 1);

    tooltip:AddLine(" ", nil, nil, nil, 1);

    local now = time();
    local diffExpires = data.expires - now;
    if (diffExpires > 0) then
        tooltip:AddDoubleLine(TIME_UNTIL_RETURNED, "|cffffffff"..SecondsToTime(diffExpires).."|r", nil, nil, nil, nil, nil, nil);
    end

    if (string.len(data.body) > 0) then
        tooltip:AddLine(" ", nil, nil, nil, 1);
        tooltip:AddLine("|cffffffff"..data.body.."|r", nil, nil, nil, 1);
    end

    if (data.money > 0) then
        tooltip:AddLine(" ", nil, nil, nil, 1);
        local amount = GetCoinTextureString(data.money);
        tooltip:AddDoubleLine(data.moneytype, "|cffffffff"..amount.."|r", nil, nil, nil, nil, nil, nil);
    end

    tooltip:AddLine(" ", nil, nil, nil, 1);

    local hasItems = nil, r, g, b, hex, color;
    if (data.items) then
        for itemName, itemData in pairs(data.items) do
            r, g, b, hex = GetItemQualityColor(itemData.rarity or -1);
            color = cui_rgb2hex(r, g, b);
            tooltip:AddDoubleLine("|cff"..color..itemName.."|r", "|cffffffff  "..itemData.count.."|r", nil, nil, nil, nil, nil, nil);
            hasItems = true;
        end
    end

    if (not hasItems) then
        tooltip:AddLine(NO_ATTACHMENTS, nil, nil, nil, 1);
    end

    tooltip:Show();
end
