local battlefieldMapFrameDone = false;

local partyMembersOnTaxi = {};

function CleanUI_InitMap()
    CleanUI_MapHookEvents();
    CleanUI_RegisterMapEvents();
    
    CleanUI_ChangeMap();

    if (BattlefieldMapFrame) then
        CleanUI_ChangeBattlefieldMap();
        battlefieldMapFrameDone = true
    end

    CleanUI_UpdateQuestHeader();
end

function CleanUI_RegisterMapEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");
    frame:RegisterEvent("QUEST_LOG_UPDATE");
    frame:RegisterEvent("CHAT_MSG_ADDON");

    frame:HookScript("OnEvent", CleanUI_MapOnEvent);

    WorldMapFrame:HookScript("OnUpdate", CleanUI_WorldMapFrame_OnUpdate);
end

function CleanUI_MapOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_BattlefieldMap" ) then
            if (not battlefieldMapFrameDone) then
                CleanUI_ChangeBattlefieldMap();
            end
        end
        if ( addonName == "Blizzard_FlightMap" ) then
            hooksecurefunc(FlightMap_WorldQuestPinMixin, "RefreshVisuals", CleanUI_WorldQuestPinMixin_RefreshVisuals);
            hooksecurefunc(MapCanvasMixin, "OnCanvasScaleChanged", CleanUI_UpdateFlightPathMap);
            hooksecurefunc("TakeTaxiNode", CleanUI_TakeTaxiNode);
            hooksecurefunc("MainMenuBarVehicleLeaveButton_Update", CleanUI_MainMenuBarVehicleLeaveButton_Update);
            FlightMapFrame:HookScript("OnShow", CleanUI_UpdateFlightPathMap);
            FlightMapFrame:HookScript("OnUpdate", CleanUI_UpdateFlightPathMap);
        end
    end

    if (event == "CHAT_MSG_ADDON") then
        local prefix, message, type, sender = ...;
        if (prefix == "CUI_SEND_FP") then
            CleanUI_ReceiveFlightPathInfo(message);
        end
    end

    if (event == "QUEST_LOG_UPDATE") then
        CleanUI_UpdateQuestHeader();
    end
end

function CleanUI_TakeTaxiNode(slotIndex)
    if ( UnitInParty("player") ) then
        local mapID = FlightMapFrame:GetMapID();
        local taxiNodes = C_TaxiMap.GetAllTaxiNodes(mapID);
        for i, taxiNodeData in ipairs(taxiNodes) do
            if taxiNodeData.slotIndex == slotIndex then
                local x, y = taxiNodeData.position:GetXY();
                local message = "IN;"..taxiNodeData.name..";"..taxiNodeData.slotIndex..";"..UnitName("player")..";"..x..";"..y;
                local success = C_ChatInfo.SendAddonMessage("CUI_SEND_FP", message, "PARTY");
            end
        end
    end
end

function CleanUI_MainMenuBarVehicleLeaveButton_Update()
    if ( CanExitVehicle() and ActionBarController_GetCurrentActionBarState() == LE_ACTIONBAR_STATE_MAIN ) then
        -- in vehicle
    else
        local message = "OUT;"..UnitName("player");
        local success = C_ChatInfo.SendAddonMessage("CUI_SEND_FP", message, "PARTY");
    end
end

function CleanUI_ReceiveFlightPathInfo(message)
    local type, name, slotIndex, sender, str_x, str_y = strsplit(";", message, 6);
    local x = tonumber(str_x);
    local y = tonumber(str_y);

    if (type == "IN") then
        if (UnitName("player") ~= sender) then
            DEFAULT_CHAT_FRAME:AddMessage(sender.." fliegt nach "..name..".", 0.7, 0.8, 1.0, 53, 10);
            partyMembersOnTaxi[sender] = {slotIndex=slotIndex, name=name, sender=sender, x=x, y=y};
            CleanUI_UpdateFlightPathMap();
        end
    else
        local copyData = {};
        for foundName, data in pairs(partyMembersOnTaxi) do
            if (name ~= foundName) then
                copyData[foundName] = data;
            end
        end
        partyMembersOnTaxi = copyData;
        CleanUI_UpdateFlightPathMap();
    end
end

function CleanUI_UpdateFlightPathMap()
    local index = 1;
    local frame, x, y;

    if (FlightMapFrame ~= nil) then
        if (FlightMapFrame:IsVisible()) then
            if (FlightMapFrame:GetCanvasZoomPercent() == 0) then
                for name, data in pairs(partyMembersOnTaxi) do
                    frame = CleanUI_GetOrCreateFlightMapMarker(index);

                    x = FlightMapFrame.ScrollContainer:GetWidth() * data.x;
                    y = FlightMapFrame.ScrollContainer:GetHeight() * -data.y;

                    frame:ClearAllPoints();
                    frame:SetPoint("CENTER", FlightMapFrame.ScrollContainer, "TOPLEFT", x, y);
                    frame:Show();

                    frame.infotext:SetText(name);

                    index = index + 1;
                end
            end
        end

        for i=index, 5 do
            frame = CleanUI_GetOrCreateFlightMapMarker(i);
            frame:Hide();
        end
    end
end

function CleanUI_GetOrCreateFlightMapMarker(index)
    local name = "CUI_FlightMapMarker"..index;

    local frame = _G[name];
    if (not frame) then
        frame = CreateFrame("Frame", name, FlightMapFrame.ScrollContainer);
        frame:SetParent(FlightMapFrame.ScrollContainer);
        frame:SetFrameStrata("HIGH");
        frame:SetSize(40, 40);

        frame.background = frame:CreateTexture("ARTWORK");
        frame.background:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0);
        frame.background:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, 0);
        frame.background:SetTexture("Interface\\Artifacts\\Artifacts-PerkRing-Final-Mask");

        frame.infotext = frame:CreateFontString(nil, "ARTWORK");
        frame.infotext:SetFontObject("GameFontNormalSmall");
        frame.infotext:SetWidth(frame:GetWidth());
        frame.infotext:SetHeight(14);
        frame.infotext:ClearAllPoints();
        frame.infotext:SetPoint("TOP", frame, "BOTTOM", 0, 8);
        frame.infotext:SetJustifyH("CENTER");
    end

    return frame;
end

function CleanUI_MapHookEvents()    
    hooksecurefunc("WorldMap_AddQuestTimeToTooltip", function(questID)
        local timeLeftMinutes = C_TaskQuest.GetQuestTimeLeftMinutes(questID) * 60;
        if timeLeftMinutes then
            _G["WorldMapTooltipTextLeft"..WorldMapTooltip:NumLines()]:SetText(BONUS_OBJECTIVE_TIME_LEFT:format(SecondsToTime(timeLeftMinutes)));
        end
    end)
    
    WorldMapFrame:HookScript("OnShow", CleanUI_WorldMapFrame_OnShow);

    hooksecurefunc (WorldMap_WorldQuestPinMixin, "RefreshVisuals", CleanUI_WorldQuestPinMixin_RefreshVisuals);
end

function CleanUI_WorldQuestPinMixin_RefreshVisuals(self)
    local inset = 0;
    local border = -9;
    if (not self.cui_texture) then
        self.cui_texture = self:CreateTexture("ARTWORK");
        self.cui_texture:SetPoint("TOPLEFT", self, "TOPLEFT", inset, -inset);
        self.cui_texture:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -inset, inset);
        self.cui_texture:SetMask("Interface\\CharacterFrame\\TempPortraitAlphaMask");
    end

    self.BountyRing:SetPoint("CENTER", self, "CENTER", 0, -2.5);
    self.BountyRing:SetSize(85, 85);

    if (not self.cui_border) then
        self.cui_border = self:CreateTexture("ARTWORK");
        self.cui_border:SetPoint("TOPLEFT", self, "TOPLEFT", border, -border);
        self.cui_border:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -border, border);
        self.cui_border:SetTexture("Interface\\CharacterFrame\\TotemBorder");
        self.cui_border:SetDrawLayer("ARTWORK", 6);
    end
    
    if (not self.cui_infotext) then
        self.cui_infotext = self:CreateFontString(nil, "ARTWORK");
        self.cui_infotext:SetFontObject("GameFontNormalHuge");
        self.cui_infotext:SetWidth(self:GetWidth());
        self.cui_infotext:SetHeight(14);
        self.cui_infotext:ClearAllPoints();
        self.cui_infotext:SetPoint("TOP", self, "BOTTOM", 0, -2);
        self.cui_infotext:SetJustifyH("CENTER");
    end

    if (not self.cui_factionMarker) then
        self.cui_factionMarker = self:CreateTexture("ARTWORK");
        self.cui_factionMarker:SetSize(27, 27);
        self.cui_factionMarker:SetPoint("CENTER", self, "TOP", 0, 0);
        self.cui_factionMarker:SetTexture("Interface\\RaidFrame\\ReadyCheck-NotReady");
        self.cui_factionMarker:SetDrawLayer("ARTWORK", 7);
    end
    
    self.cui_infotext:SetText("");

    if (self.questID) then
        local tagID, tagName, worldQuestType, rarity, isElite, tradeskillLineIndex, displayTimeLeft = GetQuestTagInfo(self.questID);
        local numQuestCurrencies = GetNumQuestLogRewardCurrencies(self.questID);
        local numQuestLogRewards = GetNumQuestLogRewards(self.questID);
        local money = GetQuestLogRewardMoney(self.questID);

        if (numQuestCurrencies >= 1) then
            local name, texture, count, currencyID = GetQuestLogRewardCurrencyInfo(1, self.questID);

            self.cui_texture:SetTexture(texture);
            self.cui_texture:Show();
            self.cui_border:Show();
            self.Texture:Hide();

            local rarity = select(8, GetCurrencyInfo(currencyID));
            if (rarity) then
                local rarityColor = ITEM_QUALITY_COLORS[rarity];
                self.cui_border:SetDesaturated(1);
                self.cui_border:SetVertexColor(rarityColor.r, rarityColor.g, rarityColor.b);
            else
                self.cui_border:SetDesaturated(nil);
                self.cui_border:SetVertexColor(1.0, 1.0, 1.0);
            end

            if (count > 1) then
                self.cui_infotext:SetText(""..count);
                self.cui_infotext:Show();
            else
                self.cui_infotext:Hide();
            end
        elseif numQuestLogRewards >= 1 then
            local name, texture, count, quality, isUsable, itemId = GetQuestLogRewardInfo(1, self.questID);
            self.cui_texture:SetTexture(texture);
            self.cui_texture:Show();
            self.cui_border:Show();
            self.Texture:Hide();

            if (quality) then
                local r, g, b = GetItemQualityColor(quality);
                self.cui_border:SetDesaturated(1);
                self.cui_border:SetVertexColor(r, g, b);
            else
                self.cui_border:SetDesaturated(nil);
                self.cui_border:SetVertexColor(1.0, 1.0, 1.0);
            end
        elseif money > 0 then
            self.cui_texture:SetTexture("Interface\\Icons\\inv_misc_coin_01");
            self.cui_texture:Show();
            self.cui_border:Show();
            self.cui_infotext:Hide();

            self.Texture:Hide();

            self.cui_border:SetDesaturated(nil);
            self.cui_border:SetVertexColor(1.0, 1.0, 1.0);
        else
            self.cui_texture:Hide();
            self.cui_border:Hide();
            self.cui_infotext:Hide();
            self.Texture:Show();

            self.cui_border:SetDesaturated(nil);
            self.cui_border:SetVertexColor(1.0, 1.0, 1.0);
        end

        local _, factionID = C_TaskQuest.GetQuestInfoByQuestID(self.questID);
        if (factionID) then
            local _, _, standingID = GetFactionInfoByID(factionID);
            if (standingID == 8) then
                self.cui_factionMarker:Show();
            else
                self.cui_factionMarker:Hide();
            end
            self.cui_factionMarker:Hide();
        end
    end
end

function CleanUI_WorldMapFrame_OnShow()
    PlayerMovementFrameFader.AddDeferredFrame(WorldMapFrame, .4, 1.0, .5, function() return GetCVarBool("mapFade") and not WorldMapFrame:IsMouseOver() end);
end

local mouseOver = true;
function CleanUI_WorldMapFrame_OnUpdate()
    if (WorldMapFrame:IsMouseOver() == mouseOver) then
        return;
    end

    if (WorldMapFrame:IsMouseOver()) then
        WorldMapFrame.BorderFrame:Show();
        WorldMapFrame.NavBar:Show();
        WorldMapFrameBg:Show();
    
        CleanUI_RemoveBackdrop(WorldMapFrame);
    else
        if ( QuestMapFrame:IsShown() ) then
            return;
        end
    
        WorldMapFrame.BorderFrame:Hide();
        WorldMapFrame.NavBar:Hide();
        WorldMapFrameBg:Hide();
    
        CleanUI_SetBackdrop(WorldMapFrame, true, nil, -63, -1, -1, 0);
    end

    mouseOver = WorldMapFrame:IsMouseOver();
end

function CleanUI_UpdateQuestHeader()
    local numQuests = tostring(select(2, GetNumQuestLogEntries()));
    local text = TRACKER_HEADER_QUESTS.."    "..numQuests.."/"..MAX_QUESTS;
    ObjectiveTrackerBlocksFrame.QuestHeader.Text:SetText(text);
end

function CleanUI_ChangeMap()
    -- Koordinaten
    local coords = WorldMapFrame:CreateFontString("CUIWorldMapFrameCoords", "OVERLAY");
    coords:SetFontObject("GameFontNormalSmall");
    coords:SetWidth(WorldMapFrame.ScrollContainer:GetWidth());
    coords:SetHeight(20);
    coords:ClearAllPoints();
    coords:SetPoint("BOTTOMLEFT", WorldMapFrame.ScrollContainer, "BOTTOMLEFT", 0, -20);
    coords:SetPoint("BOTTOMRIGHT", WorldMapFrame.ScrollContainer, "BOTTOMRIGHT", 0, -20);
    coords:SetJustifyH("CENTER");
    coords:SetText("Hallo");

    local refFrame = CreateFrame("Frame", "CUI_WorldMapFrameCoordsFrame", WorldMapFrame);
    refFrame:HookScript("OnUpdate", CleanUI_UpdateMapCoords);
end

function CleanUI_UpdateMapCoords()
    if (not WorldMapFrame:IsVisible()) then
        return;
    end

    -- player
    local px, py = CleanUI_GetPlayerMapPosition();
    
    -- cursor
    local adjustedX, adjustedY = CleanUI_GetCursorPosition()

    local output = UnitName("player")..": "..px.."|cffffffff/|r"..py;

    if (adjustedX and adjustedY) then
        output = output.." -- Cursor: "..adjustedX.."|cffffffff/|r"..adjustedY;
    end

    CUIWorldMapFrameCoords:SetText(output);
end

function CleanUI_GetPlayerMapPosition()
    local mapID = C_Map.GetBestMapForUnit("player");

    if (mapID and C_Map.GetPlayerMapPosition(mapID, "player")) then
        local x, y = C_Map.GetPlayerMapPosition(mapID, "player"):GetXY();
        
        x = x or 0;
        y = y or 0;
    
        x = x * 100;
        y = y * 100;
    
        return "|cffffffff"..string.format("%.1f", x).."|r", "|cffffffff"..string.format("%.1f", y).."|r";
    end

    return "|cffffffff0|r", "|cffffffff0|r";
end

function CleanUI_GetCursorPosition()
    local adjustedX, adjustedY = WorldMapFrame:GetNormalizedCursorPosition();

    adjustedX = adjustedX * 100;
    adjustedY = adjustedY * 100;

    if (adjustedX == 0 or adjustedY == 0 or adjustedX >= 100 or adjustedY >= 100) then
        return nil, nil, 0, 0;
    end

    return "|cffffffff"..string.format("%.1f", adjustedX).."|r", "|cffffffff"..string.format("%.1f", adjustedY).."|r", adjustedX, adjustedY;
end

function CleanUI_ChangeBattlefieldMap()
    BattlefieldMapFrame.BorderFrame:Hide();
    CleanUI_SetBackdrop(BattlefieldMapFrame, false, nil, 5, -3, 0, 1);
end