petTypeFilterButtons = {};

-- saved data
CleanUIBattlePetGroups = {};
local petStats = {};
local selectedIndex = nil;

StaticPopupDialogs["CUI_BATTLEPET_GROUPS_ADD_TITLE_POPUP"] = {
    text = NAME,
    button1 = SAVE,
    button2 = CANCEL,
    OnShow = function (self, data)
        self.editBox:SetText(data or "nil")
    end,
    OnAccept = function (self, data, data2)
        CleanUI_AddPetBattleGroupWithName(self.editBox:GetText())
    end,
    hasEditBox = true,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
}

StaticPopupDialogs["CUI_BATTLEPET_GROUPS_RENAME_TITLE_POPUP"] = {
    text = BATTLE_PET_RENAME,
    button1 = SAVE,
    button2 = CANCEL,
    OnShow = function (self, data)
        self.editBox:SetText(data or "nil")
    end,
    OnAccept = function (self, data, data2)
        CleanUI_RenamePetBattleGroupWithName(self.editBox:GetText())
    end,
    hasEditBox = true,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
}

function CleanUI_InitPets()
    CleanUI_RegisterPetsEvents();
end

function CleanUI_RegisterPetsEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");
    frame:RegisterEvent("PET_JOURNAL_PETS_HEALED");
    frame:RegisterEvent("PET_JOURNAL_PET_RESTORED");
    frame:RegisterEvent("PET_JOURNAL_PET_REVOKED");

    frame:HookScript("OnEvent", CleanUI_PetsOnEvent);
end

function CleanUI_PetsOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_Collections") then
            CleanUI_Pets_ChangeUI();
            CleanUI_PetsHookEvents();
            CleanUI_InitPetBattleGroups();
        end
    end
    if (event == "PET_JOURNAL_PETS_HEALED" or event == "PET_JOURNAL_PET_RESTORED" or event == "PET_JOURNAL_PET_REVOKED") then
        CleanUI_UpdatePetBattleGroups();
    end
end

function CleanUI_PetsHookEvents()   
    hooksecurefunc(C_PetJournal, "SetPetTypeFilter", CleanUI_Pets_UpdateFilterButtons);
    hooksecurefunc(C_PetJournal, "SetAllPetTypesChecked", CleanUI_Pets_UpdateFilterButtons);

    hooksecurefunc("PetJournal_UpdatePetList", CleanUI_PetJournal_UpdatePetList);

    PetJournalListScrollFrame:HookScript("OnVerticalScroll", CleanUI_PetJournal_UpdatePetList)
    PetJournalListScrollFrame:HookScript("OnMouseWheel", CleanUI_PetJournal_UpdatePetList)
end

------------------------------------------------------------------------------------------------
-- FILTER BUTTONS
------------------------------------------------------------------------------------------------

local function createFilterButton(id, type)
    local filterButton = CreateFrame("CheckButton", nil, PetJournal.LeftInset, "SecureFrameTemplate");
    filterButton:SetSize(22, 22);
    filterButton:SetNormalTexture("Interface\\ICONS\\Icon_PetFamily_"..PET_TYPE_SUFFIX[id]);
    filterButton:GetNormalTexture():SetTexCoord(0.08, 0.92, 0.08, 0.92);
    filterButton:SetPoint("BOTTOMLEFT", PetJournal.listScroll, "TOPLEFT", 25 * (id - 1) + 3, 4);
    filterButton:SetAlpha(1);
    filterButton:SetID(id);

    filterButton:HookScript("OnEnter", CleanUI_Pets_OnButtonEnter);
    filterButton:HookScript("OnLeave", CleanUI_Pets_OnButtonLeave);

    filterButton:SetScript("OnClick", CleanUI_Pets_FilterClicked);

    return filterButton;
end

local function petBattlesCheatSheet(id)
    if (id == 1) then return 2, 5, 8, 4; end
    if (id == 2) then return 6, 3, 4, 1; end
    if (id == 3) then return 9, 8, 2, 6; end
    if (id == 4) then return 1, 2, 9, 5; end
    if (id == 5) then return 4, 7, 1, 8; end
    if (id == 6) then return 3, 9, 10, 2; end
    if (id == 7) then return 10, 10, 5, 9; end
    if (id == 8) then return 5, 1, 3, 10; end
    if (id == 9) then return 7, 4, 6, 3; end
    if (id == 10) then return 8, 6, 7, 7; end

    return 1, 1, 1, 1;
end

function CleanUI_Pets_OnButtonEnter(self)
    GameTooltip_SetDefaultAnchor(GameTooltip, self);
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");

    GameTooltip:SetText(_G["BATTLE_PET_NAME_"..self:GetID()], 1.0, 1.0, 1.0);

    local bonusStrike, bonusDefense, weakStrike, weakDefense = petBattlesCheatSheet(self:GetID());
    
    GameTooltip:AddLine(" ");
    GameTooltip:AddLine("Use Against", 0.0, 1.0, 0.0);
    GameTooltip:AddDoubleLine(_G["BATTLE_PET_NAME_"..bonusStrike], "Bonus Strike (+50%)", 1.0, 1.0, 1.0, 0.0, 1.0, 0.0);
	GameTooltip:AddTexture("Interface\\ICONS\\Icon_PetFamily_"..PET_TYPE_SUFFIX[bonusStrike]);
    GameTooltip:AddDoubleLine(_G["BATTLE_PET_NAME_"..bonusDefense], "Bonus Defense (+33%)", 1.0, 1.0, 1.0, 0.0, 1.0, 0.0);
    GameTooltip:AddTexture("Interface\\ICONS\\Icon_PetFamily_"..PET_TYPE_SUFFIX[bonusDefense]);
    
    GameTooltip:AddLine(" ");
    GameTooltip:AddLine("Don't Use Against", 1.0, 0.0, 0.0);
    GameTooltip:AddDoubleLine(_G["BATTLE_PET_NAME_"..weakStrike], "Weak Strike (-33%)", 1.0, 1.0, 1.0, 1.0, 0.0, 0.0);
	GameTooltip:AddTexture("Interface\\ICONS\\Icon_PetFamily_"..PET_TYPE_SUFFIX[weakStrike]);
    GameTooltip:AddDoubleLine(_G["BATTLE_PET_NAME_"..weakDefense], "Weak Defense (-50%)", 1.0, 1.0, 1.0, 1.0, 0.0, 0.0);
	GameTooltip:AddTexture("Interface\\ICONS\\Icon_PetFamily_"..PET_TYPE_SUFFIX[weakDefense]);

    GameTooltip:Show();
end

function CleanUI_Pets_OnButtonLeave(self)
    GameTooltip:FadeOut();
end

function CleanUI_Pets_ChangeUI()  
    if (PetJournal.cui_done) then
        return;
    end

    PetJournal.listScroll:SetPoint("TOPLEFT", PetJournal.LeftInset, "TOPLEFT", 3, -63);

    local button;
    for index = 1, 10 do
        button = createFilterButton(index);
        tinsert(petTypeFilterButtons, button);
    end

    PetJournal.cui_done = true;
    
    CleanUI_Pets_UpdateFilterButtons();
end

local function updateFilterButtonAlpha(button)
    local alpha = 1.0;
    if (not C_PetJournal.IsPetTypeChecked(button:GetID())) then
        alpha = 0.3;
    end

    button:SetAlpha(alpha);
end

function CleanUI_Pets_UpdateFilterButtons()
    if (not PetJournal.cui_done) then
        return;
    end

    for i, button in ipairs(petTypeFilterButtons) do
        updateFilterButtonAlpha(button);
    end
end

function CleanUI_Pets_FilterClicked(self)
    local checked = C_PetJournal.IsPetTypeChecked(self:GetID());
    C_PetJournal.SetPetTypeFilter(self:GetID(), not checked);

    if (IsAltKeyDown()) then
        for index = 1, 10 do
            C_PetJournal.SetPetTypeFilter(index, not checked);
        end
    end
end

function CleanUI_PetJournal_UpdatePetList()
    CleanUI_PetJournal_UpdatePetCount();
    CleanUI_PetJournal_UpdateCompanionListButtons();

    local scrollFrame = PetJournal.listScroll;
    local offset = HybridScrollFrame_GetOffset(scrollFrame);
    local petButtons = scrollFrame.buttons;
    local pet, index;

    local numPets, numOwned = C_PetJournal.GetNumPets();

    for i = 1,#petButtons do
        pet = petButtons[i];
        pet.name:SetWidth(125);

        if (not pet.cuihealth) then
            pet.cuihealth = CreateFrame("Frame", pet:GetName().."CUI_HEATH", pet, "CleanUIPetInfoHealthTemplate");
            --pet.cuihealth:SetFrameStrata("MEDIUM");
            pet.cuihealth:SetPoint("TOPRIGHT", pet, "TOPRIGHT", -10, -5);
            pet.cuihealth:SetScale(0.75);
        end

        index = offset + i;
        if index <= numPets then
            local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle = C_PetJournal.GetPetInfoByIndex(index);
            if isOwned then
                local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID);
                local speciesID, customName, level, xp, maxXp, displayID, isFavorite, name, icon, petType, creatureID = C_PetJournal.GetPetInfoByPetID(petID);

                if (health < maxHealth) then
                    pet.cuihealth:Show();
                    pet.cuihealth.healthValue:SetFormattedText(PET_BATTLE_CURRENT_HEALTH_FORMAT, health, maxHealth);
                    pet.cuihealth.healthBar:SetMinMaxValues(0, maxHealth);
                    pet.cuihealth.healthBar:SetValue(health);
                else
                    pet.cuihealth:Hide();
                end
            else
                pet.cuihealth:Hide();
            end
        end
    end
end

function CleanUI_PetJournal_UpdatePetCount()
    local pets = {};

    petStats = {};

    local numUnique = 0;

    local numPets, numOwned = C_PetJournal.GetNumPets();

    local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle;
    local health, maxHealth, attack, speed, rarity;
    for i = 1, numPets do
        petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle = C_PetJournal.GetPetInfoByIndex(i);
        if (isOwned) then
            health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID);

            petStats[rarity] = (petStats[rarity] or 0) + 1;

            if (speciesID and not pets[speciesID]) then
                numUnique = numUnique + 1;
                pets[speciesID] = speciesID;
            end
        else
            petStats[0] = (petStats[0] or 0) + 1;
        end
    end

    PetJournal.PetCount.Count:SetText(numOwned.." ("..numUnique..")");
end

function CleanUI_PetJournal_UpdateCompanionListButtons()
    local scrollFrame = PetJournal.listScroll;
    local offset = HybridScrollFrame_GetOffset(scrollFrame);
    local petButtons = scrollFrame.buttons;
    local pet, index;

    local numPets, numOwned = C_PetJournal.GetNumPets();

    local summonedPetID = C_PetJournal.GetSummonedPetGUID();

    local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle;
    local health, maxHealth, attack, speed, rarity;
    local border;
    for i = 1,#petButtons do
        pet = petButtons[i];
        
        index = offset + i;
        if index <= numPets then
            petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle = C_PetJournal.GetPetInfoByIndex(index);

            if (isOwned) then
                health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID);
                pet.name:SetTextColor(ITEM_QUALITY_COLORS[rarity-1].r, ITEM_QUALITY_COLORS[rarity-1].g, ITEM_QUALITY_COLORS[rarity-1].b);
            else
                pet.name:SetTextColor(0.9, 0.9, 0.0);
            end
        end
    end
end

------------------------------------------------------------------------------------------------
-- PET BATTLE GROUPS
------------------------------------------------------------------------------------------------

function CleanUI_InitPetBattleGroups()
    local button = CreateFrame("Button", "CUIPetBattleGroupsButton", PetJournal, "UIPanelButtonTemplate");
    button:SetWidth(100);
    button:SetHeight(22);
    button:SetPoint("TOPRIGHT", PetJournal, "TOPRIGHT", -6, -230);
    button:SetText(GROUPS);
    button:SetScript("OnClick", CleanUI_TogglePetBattleGroups);
    button:SetFrameLevel(button:GetFrameLevel() + 1);

    local groupsFrame = CreateFrame("Frame", "CleanUIBattlePetGroupsFrame", PetJournal, "CleanUIBattlePetGroupsFrameTemplate");
    
    CleanUI_SetArtBackdrop(groupsFrame, 0, 0, 0, 5);

    groupsFrame:SetSize(265, 606);
    groupsFrame:ClearAllPoints();
    groupsFrame:SetPoint("BOTTOMLEFT", PetJournal, "BOTTOMRIGHT", 4, 0);

    -- create gui
    HybridScrollFrame_OnLoad(CleanUIBattlePetGroupsFrame.scrollFrame);
    CleanUIBattlePetGroupsFrame.scrollFrame.update = CleanUI_BattlePetGroups_OnUpdate;
    HybridScrollFrame_CreateButtons(CleanUIBattlePetGroupsFrame.scrollFrame, "CleanUIBattlePetGroupsEntryTemplate");
end

function CleanUI_TogglePetBattleGroups()
    if (CleanUIBattlePetGroupsFrame:IsVisible()) then
        CleanUIBattlePetGroupsFrame:Hide();
    else
        CleanUIBattlePetGroupsFrame:Show();
        CleanUI_BattlePetGroups_OnUpdate();
    end
end

function CleanUI_UpdatePetBattleGroups()
    if (CleanUIBattlePetGroupsFrame:IsVisible()) then
        CleanUI_BattlePetGroups_OnUpdate();
    end
end

function CleanUI_BattlePetGroups_OnUpdate()
    if (not CleanUIBattlePetGroupsFrame) then
        return;
    end

    --update ui stats
    local groupsActive = 1;
    for p=1, 3 do
        _, _, _, _, locked = C_PetJournal.GetPetLoadOutInfo(p);
        if (locked) then
            groupsActive = nil;
        end
    end

    if (groupsActive) then
        CUIPetBattleGroupsButton:Enable();
    else
        CUIPetBattleGroupsButton:Disable();
    end

    -- update group frame
    CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:Hide();

    local buttons = CleanUIBattlePetGroupsFrame.scrollFrame.buttons;

    local numEntries = #CleanUIBattlePetGroups;
    local numButtons = #buttons;


    local scrollOffset = HybridScrollFrame_GetOffset(CleanUIBattlePetGroupsFrame.scrollFrame);
    local buttonHeight = buttons[1]:GetHeight();
    local displayedHeight = 0;

    local entry;
    local data;

    for i=1, numButtons do
        local entryIndex = i + scrollOffset;
        entry = buttons[i];

        if ( entryIndex <= numEntries ) then
            data = CleanUIBattlePetGroups[entryIndex];

            entry:Show();
            entry.index = entryIndex;

            -- button data
            entry:SetText(data.name);

            -- pet data
            local id, health, maxHealth, power, speed, rarity;
            local speciesID, customName, level, xp, maxXp, displayID, isFavorite, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable, unique, obtainable;
            local pet;
            for p=1, 3 do
                id = data.pet[p].petGUID;
                
                if (id) then
                    health, maxHealth, power, speed, rarity = C_PetJournal.GetPetStats(id);

                    speciesID, customName, level, xp, maxXp, displayID, isFavorite, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable, unique, obtainable = C_PetJournal.GetPetInfoByPetID(id);
    
                    pet = _G[entry:GetName().."Pet"..p];
    
                    rarity = rarity or 1;
                
                    pet.PetInfo.icon:SetTexture(icon);
                    pet.PetInfo.qualityBorder:SetDesaturated(nil);
                    pet.PetInfo.qualityBorder:SetVertexColor(ITEM_QUALITY_COLORS[rarity-1].r, ITEM_QUALITY_COLORS[rarity-1].g, ITEM_QUALITY_COLORS[rarity-1].b);

                    if (level < 25) then
                        pet.PetInfo.level:SetText(level);
                        pet.PetInfo.level:Show();
                        pet.PetInfo.levelBG:Show();
                    else
                        pet.PetInfo.level:Hide();
                        pet.PetInfo.levelBG:Hide();
                    end

                    if (isFavorite) then
                        pet.PetInfo.favorite:Show();
                    else
                        pet.PetInfo.favorite:Hide();
                    end
    
                    pet.PetInfo.typeIcon:SetTexture(GetPetTypeTexture(petType));
                    pet.PetInfo.isDead:SetShown((health or 0) <= 0);
    
                    -- tooltip info
                    pet.speciesName = name;
                    pet.sourceText = sourceText;
                    pet.tradable = tradable;
                    pet.unique = unique;
    
                    if ( description and description ~= "" ) then
                        pet.description = format([["%s"]], description);
                    else
                        pet.description = nil;
                    end
                end
            end

            -- this isn't a header, hide the header textures
            entry:SetNormalTexture("");
            entry:SetHighlightTexture("");

            if (entryIndex == selectedIndex) then
                -- reposition highlight frames
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:SetParent(entry);
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:ClearAllPoints();
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:SetPoint("TOPLEFT", entry, "TOPLEFT", 0, -5);
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:SetPoint("BOTTOMRIGHT", entry, "BOTTOMRIGHT", 15, -2);
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:SetFrameStrata("BACKGROUND");
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:SetAlpha(0.4);
                CleanUIBattlePetGroupsFrame.scrollFrame.highlightFrame:Show();
            end
        else
            entry:Hide();
        end

        displayedHeight = displayedHeight + buttonHeight;
    end

    HybridScrollFrame_Update(CleanUIBattlePetGroupsFrame.scrollFrame, numEntries * buttonHeight, displayedHeight);
end

function CleanUI_BattlePetGroupsPet_OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 0, 0);
    GameTooltip:SetText(self.speciesName, 1, 1, 1);
    GameTooltip:AddLine(self.sourceText, 1, 1, 1, true);
    if ( not self.tradable ) then
        GameTooltip:AddLine(BATTLE_PET_NOT_TRADABLE, 1, 0.1, 0.1, true);
    end
    if ( self.unique ) then
        GameTooltip:AddLine(ITEM_UNIQUE, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, true);
    end
    if ( self.description and self.description ~= "" ) then
        GameTooltip:AddLine(" ");
        GameTooltip:AddLine(self.description, nil, nil, nil, true);
    end
    GameTooltip:Show();
end

function CleanUI_BattlePetGroupsPet_OnLeave(self)
    GameTooltip:FadeOut();
end

function CleanUI_BattlePetGroupsEntry_OnClick(self, button, down)
    selectedIndex = self.index;
    CleanUI_BattlePetGroups_OnUpdate();

    local data = CleanUIBattlePetGroups[selectedIndex];

    if (button == "LeftButton") then
    elseif (button == "RightButton") then
        local dialog = StaticPopup_Show("CUI_BATTLEPET_GROUPS_RENAME_TITLE_POPUP");
        local editbox = _G[dialog:GetName().."EditBox"];
        editbox:SetText(data.name or "");
    end
end

function CleanUI_BattlePetGroupsEntry_OnDoubleClick(self)
    if (not selectedIndex or selectedIndex > #CleanUIBattlePetGroups) then
        return;
    end

    local data = CleanUIBattlePetGroups[selectedIndex];

    for i=1, 3 do
        C_PetJournal.SetPetLoadOutInfo(i, data.pet[i].petGUID);
        C_PetJournal.SetAbility(i, 1, data.pet[i].ability1);
        C_PetJournal.SetAbility(i, 2, data.pet[i].ability2);
        C_PetJournal.SetAbility(i, 3, data.pet[i].ability3);
    end

    PetJournal_UpdateAll();
end

function CleanUI_BattlePetGroups_ResetSelection()
    selectedIndex = nil;
end

function CleanUI_AddPetBattleGroup()
    local dialog = StaticPopup_Show("CUI_BATTLEPET_GROUPS_ADD_TITLE_POPUP");
    local editbox = _G[dialog:GetName().."EditBox"];
    editbox:SetText("");
end

function CleanUI_AddPetBattleGroupWithName(name)
    local data = {};
    data.pet = {};

    data.name = name;

    CleanUI_UpdateBattlePetGroupData(data);

    tinsert(CleanUIBattlePetGroups, data);

    CleanUI_BattlePetGroups_OnUpdate();
end

function CleanUI_RenamePetBattleGroupWithName(name)
    if (not selectedIndex or selectedIndex > #CleanUIBattlePetGroups) then
        return;
    end

    local data = CleanUIBattlePetGroups[selectedIndex];

    data.name = name;

    CleanUI_BattlePetGroups_OnUpdate();
end

function CleanUI_SavePetBattleGroup()
    if (not selectedIndex or selectedIndex > #CleanUIBattlePetGroups) then
        return;
    end

    local data = CleanUIBattlePetGroups[selectedIndex];

    CleanUI_UpdateBattlePetGroupData(data);

    CleanUI_BattlePetGroups_OnUpdate();
end

function CleanUI_DeletePetBattleGroup()
    if (not selectedIndex or selectedIndex > #CleanUIBattlePetGroups) then
        return;
    end

    tremove(CleanUIBattlePetGroups, selectedIndex);

    CleanUI_BattlePetGroups_ResetSelection();
    CleanUI_BattlePetGroups_OnUpdate();
end


function CleanUI_UpdateBattlePetGroupData(data)
    local petGUID, ability1, ability2, ability3, locked;
    local petdata;
    for i=1, 3 do
        petGUID, ability1, ability2, ability3, locked = C_PetJournal.GetPetLoadOutInfo(i);
        data.pet[i] = {petGUID = petGUID, ability1 = ability1, ability2 = ability2, ability3 = ability3};
    end
end