local raidOpacity = 0.3 -- Set the visibility, 0 being invisible
local raid_frame = CreateFrame("Frame");

local groupsAndMembersArray = {}
local raidMembersArray = {}

function CleanUI_InitUnitFrames()

    CleanUIUserData.scaleUnitFrames = CleanUIUserData.scaleUnitFrames or 1.2
    CleanUIUserData.scalePartyFrames = CleanUIUserData.scalePartyFrames or 1.2
    CleanUIUserData.dragon = CleanUIUserData.dragon or 2

    if (CleanUIUserData.dragon > 3) then
        CleanUI_SetUnitFrameDragon(2);
        CleanUI_SetUnitFrameColoredDragon(true);
    end

    CleanUI_UnitFramesHookEvents();
    CleanUI_RegisterUnitFramesEvents();
    
    CleanUI_ScaleUnitFrames();
    CleanUI_ScalePartyFrames();
    CleanUI_UpdateAlternateManaBar();
    CleanUI_UpdateUnitFrameDragon();

    CleanUI_FixUnitFrameScales();
    
    CleanUI_InitRaid();
    
    if (CompactRaidFrameManager) then
        CleanUI_UnitFramesHookRaidFrameManager();
        CleanUI_InitRaidFrameToggleButton();
    end
    
    -- aggro bar in player unit frame name area
    local aggroBar = CreateFrame("StatusBar", "PlayerFrameAggroBar", PlayerFrame, "CuiTextStatusBarTemplate");
    aggroBar:ClearAllPoints();
    aggroBar:SetPoint("TOPLEFT", PlayerFrameHealthBar, "TOPLEFT", 0, 18);
    aggroBar:SetPoint("BOTTOMRIGHT", PlayerFrameHealthBar, "TOPRIGHT", 0, -1);
    aggroBar:SetStatusBarColor(1.0, 0.0, 0.0);
    aggroBar:SetFrameStrata("LOW");
    
    aggroBar.background = aggroBar:CreateTexture("ARTWORK");
    aggroBar.background:SetPoint("TOPLEFT", aggroBar, "TOPLEFT", 0, 0);
    aggroBar.background:SetPoint("BOTTOMRIGHT", aggroBar, "BOTTOMRIGHT", 0, 0);
    aggroBar.background:SetColorTexture(0.6, 0, 0, 0.3)
    aggroBar.background:Hide();
    
    PlayerFrameBackground:SetParent(aggroBar);
    PlayerFrameBackground:SetDrawLayer("BACKGROUND");
    PlayerFrameBackground:ClearAllPoints();
    PlayerFrameBackground:SetAllPoints(aggroBar);
    
    -- colors
    CleanUI_UpdatePlayerColors();
    CleanUI_UpdatePetColors();
    CleanUI_UpdatePartyFrameColors();

    PetFrame:SetPoint("TOPLEFT", 60, -80);

    -- buff tooltip
    PartyMemberBuffTooltip:SetBackdrop(nil);
    CleanUI_SetBackdrop(PartyMemberBuffTooltip, true, nil, -3, 3, 3, -3);
end

function CleanUI_FixUnitFrameScales()
    if (RuneFrame) then
        RuneFrame:SetFrameStrata("HIGH");
        RuneFrame:SetScale(0.9);
        RuneFrame:ClearAllPoints();
        RuneFrame:SetPoint("TOPRIGHT", PlayerFrame, "BOTTOMRIGHT", 0, 35);
    end
end

function CleanUI_InitRaid()
    raid_frame:SetScript("OnUpdate", CleanUI_RaidOnUpdate);
    for i = 1,8 do
      groupsAndMembersArray[i] = {}
      for j = 1,5 do
         groupsAndMembersArray[i][j] = string.format("CompactRaidGroup%dMember%d", i,j)
      end
    end
    
    local raidMembersArray = {}
    for i = 1,40 do
      raidMembersArray[i] = string.format("CompactRaidFrame%d", i) 
    end
end

function CleanUI_RegisterUnitFramesEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");
    
    frame:RegisterEvent("PLAYER_ENTER_COMBAT");
    
    frame:RegisterEvent("RAID_ROSTER_UPDATE");
    frame:RegisterEvent("RAID_INSTANCE_WELCOME");
    
    frame:RegisterEvent("PLAYER_REGEN_DISABLED");
    frame:RegisterEvent("PLAYER_REGEN_ENABLED");
    
    frame:RegisterEvent("UNIT_PET");

    frame:HookScript("OnEvent", CleanUI_UnitFramesOnEvent);
end

function CleanUI_UnitFramesOnEvent(self, event, ...)

    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_CompactRaidFrames" ) then
            CleanUI_UnitFramesHookRaidFrameManager();
            CleanUI_InitRaidFrameToggleButton()
        end
    end

    if (event == "RAID_ROSTER_UPDATE") then
        CleanUI_ScalePartyFrames();
    end

    if (event == "PLAYER_REGEN_DISABLED" or event == "PLAYER_REGEN_ENABLED" or event == "PLAYER_ENTER_COMBAT") then
        CleanUI_UpdatePlayerColors();
 
        if (InCombatLockdown()) then
            PlayerFrameAggroBar.background:Show();
        else
            PlayerFrameAggroBar.background:Hide();
        end
    end

    if (event == "UNIT_PET") then
        CleanUI_UpdatePetColors();
    end
    
    CleanUI_UpdatePartyFrameColors();
end

function CleanUI_UnitFramesHookEvents()
end

function CleanUI_UnitFramesHookRaidFrameManager()
    CleanUI_CompactRaidFrameManager_Collapse(CompactRaidFrameManager);

    hooksecurefunc("CompactRaidFrameManager_Expand", CleanUI_CompactRaidFrameManager_Expand);
    hooksecurefunc("CompactRaidFrameManager_Collapse", CleanUI_CompactRaidFrameManager_Collapse);
    
    hooksecurefunc("TargetFrame_CheckFaction", CleanUI_TargetFrame_CheckFaction);
    hooksecurefunc("UnitFrame_UpdateThreatIndicator", CleanUI_UnitFrame_UpdateThreatIndicator);
    hooksecurefunc("UnitFrame_SetUnit", CleanUI_UpdatePartyFrameColors);
    hooksecurefunc("TargetofTarget_Update", CleanUI_UpdateTargetofTargetColors);
    
    hooksecurefunc("BossTargetFrame_OnLoad", CleanUI_BossTargetFrame_OnLoad);
    hooksecurefunc("CompactRaidFrameContainer_LayoutFrames", CleanUI_CompactRaidFrameContainer_LayoutFrames);
end

local UPDATE_WAIT = 0.05;
local wait = 0;
function CleanUI_RaidOnUpdate(self, elapsed)
    wait = wait + elapsed;

    if (wait > UPDATE_WAIT) then
        wait = 0;
        CleanUI_RaidDoUpdate(self);
    end
end

local function set_opacity(raidmember)
    if raidmember then
        local transparency;
    
        if raidmember:GetAlpha() < 0.8 then
            transparency = raidOpacity;
        else
            transparency = 1;
        end
        
        select(1,raidmember:GetRegions()):SetAlpha(transparency); -- Background
        select(7,raidmember:GetRegions()):SetAlpha(transparency); -- Name Text
        select(1,raidmember:GetChildren()):SetAlpha(transparency); -- Health Bar
        select(2,raidmember:GetChildren()):SetAlpha(transparency); -- Mana Bar
    end
end

function CleanUI_RaidDoUpdate(self)
    --Keep Groups Together Unchecked
    for i=1, 40 do
        local raidmember = _G[raidMembersArray[i]];
        set_opacity(raidmember);
    end
    
      -- Keep Groups Together Checked
    for i=1, 8 do
        for j=1,5 do
            local raidmember = _G[groupsAndMembersArray[i][j]];
            set_opacity(raidmember);
        end
    end
end

function CleanUI_InitRaidFrameToggleButton()
    local button = _G["CUIRaidFrameToggleButton"];
    if (not button) then
        button = CreateFrame("Button", "CUIRaidFrameToggleButton", CompactRaidFrameManager, "UIPanelButtonTemplate");
        CleanUI_RemoveUiButtonTexture(button);
        button:SetSize(12, 12);
        button:SetNormalTexture("Interface\\RaidFrame\\RaidPanel-Toggle");
        button:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
        button:SetPoint("BOTTOMRIGHT", CompactRaidFrameManager, "TOPRIGHT", -6, -3);

        button:SetScript("OnClick", CleanUI_ToggleRaidFrame);
    end

    CleanUI_ToggleRaidFrame();
    CleanUI_UpdateRaidFrameToggleButtonText();
end

local raidFramesHidden = true;

function CleanUI_ToggleRaidFrame()

    CompactRaidFrameContainer:ClearAllPoints();

    if (raidFramesHidden) then
        CompactRaidFrameContainer:SetPoint("TOPLEFT", CompactRaidFrameManager, "TOPRIGHT", 5, -10);
    else
        CompactRaidFrameContainer:SetPoint("TOPLEFT", CompactRaidFrameManager, "TOPRIGHT", -10000, 0);
    end
    
    raidFramesHidden = not raidFramesHidden;

    CleanUI_UpdateRaidFrameToggleButtonText();
end

function CleanUI_UpdateRaidFrameToggleButtonText()
    local button = _G["CUIRaidFrameToggleButton"];

    if (raidFramesHidden) then
      --button:SetText("-");
      button:GetNormalTexture():SetTexCoord(0.25, 0.5, 0.4, 0.6);
    else
      --button:SetText("+");
      button:GetNormalTexture():SetTexCoord(0.75, 1, 0.4, 0.6);
    end

    if (IsInRaid()) then
        button:Show();
    else
        button:Hide();
    end
end

function CleanUI_CompactRaidFrameContainer_LayoutFrames(self)
    CleanUI_UpdateRaidFrameToggleButtonText();
end

function CleanUI_UpdatePlayerColors()
    PlayerName:SetTextColor(1.0, 1.0, 1.0, 1.0);
    
    if (InCombatLockdown()) then
        PlayerName:SetTextColor(1.0, 0.0, 0.0, 1.0);
    end
end

function CleanUI_BossTargetFrame_OnLoad(self, unit, event)
    local classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
    
    if (unit) then
        local localizedClass, englishClass = UnitClass(unit);
        classColor = RAID_CLASS_COLORS[englishClass];
    
        if (not classColor) then
            classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
        end
    end
    
    self.borderTexture:SetDesaturated(1);
    self.borderTexture:SetVertexColor(classColor.r, classColor.g, classColor.b);
end

function CleanUI_UpdatePartyFrameColors()
    local partyFrame, partyFrameName, partyFrameTexture;
    for i=1, 5 do
        partyFrame = _G["PartyMemberFrame"..i];
        partyFrameName = _G["PartyMemberFrame"..i.."Name"];
        partyFrameTexture = _G["PartyMemberFrame"..i.."Texture"];
        
        if (partyFrame) then
        
            local classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
            local unit = partyFrame.unit;
            
            if (unit) then
                local localizedClass, englishClass = UnitClass(unit);
                classColor = RAID_CLASS_COLORS[englishClass];
            
                if (not classColor) then
                    classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
                end
            end
            
            partyFrameName:SetTextColor(classColor.r, classColor.g, classColor.b, 1.0);
            
            partyFrameTexture:SetDesaturated(1);
            partyFrameTexture:SetVertexColor(classColor.r, classColor.g, classColor.b);
        end
    end
end

function CleanUI_UpdatePetColors()
    local classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
    local unit = "pet";
    
    if (unit) then
        local localizedClass, englishClass = UnitClass(unit);
        classColor = RAID_CLASS_COLORS[englishClass];
    
        if (not classColor) then
            classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
        end
    end
            
    PetFrameTexture:SetDesaturated(1);
    PetFrameTexture:SetVertexColor(classColor.r, classColor.g, classColor.b);
end

function CleanUI_UpdateTargetofTargetColors(self, elapsed)
    if (not UnitExists(self.unit)) then
        return;
    end
    
    local texture = _G[self:GetName().."TextureFrameTexture"];
    
    local localizedClass, englishClass = UnitClass(self.unit);
    local classColor = RAID_CLASS_COLORS[englishClass];

    if (not classColor) then
        classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
    end
            
    texture:SetDesaturated(1);
    texture:SetVertexColor(classColor.r, classColor.g, classColor.b);
end

function CleanUI_UpdateAlternateManaBar()
    if (not PlayerFrameAlternateManaBar) then
        return;
    end

    PlayerFrameAlternateManaBar.MonkBackground:Hide();
    PlayerFrameAlternateManaBar.DefaultBorder:Hide();
    PlayerFrameAlternateManaBar.DefaultBorderLeft:Hide();
    PlayerFrameAlternateManaBar.DefaultBorderRight:Hide();
    PlayerFrameAlternateManaBar.MonkBorder:Hide();
    
    if (not PlayerFrameAlternateManaBar.cuiborder) then
        PlayerFrameAlternateManaBar.cuiborder = PlayerFrameAlternateManaBar:CreateTexture("OVERLAY");
        PlayerFrameAlternateManaBar.cuiborder:SetPoint("TOPLEFT", PlayerFrameAlternateManaBar, "TOPLEFT", -2, 2);
        PlayerFrameAlternateManaBar.cuiborder:SetPoint("BOTTOMRIGHT", PlayerFrameAlternateManaBar, "BOTTOMRIGHT", 2, -2);
        PlayerFrameAlternateManaBar.cuiborder:SetTexture("Interface\\Tooltips\\UI-StatusBar-Border");
        PlayerFrameAlternateManaBar.cuiborder:SetDrawLayer("OVERLAY", 7);
    end
    
    PlayerFrameAlternateManaBar:ClearAllPoints();
    
    PlayerFrameAlternateManaBar:SetPoint("TOPLEFT", PlayerFrameManaBar, "BOTTOMLEFT", 15, -5);
    PlayerFrameAlternateManaBar:SetPoint("BOTTOMRIGHT", PlayerFrameManaBar, "BOTTOMRIGHT", -15, -13);
end

function CleanUI_UnitFrame_UpdateThreatIndicator(indicator, numericIndicator, unit)
    if ( not indicator ) then
        return;
    end
    
    local isTanking, status, threatpct, rawthreatpct, threatvalue = UnitDetailedThreatSituation("player", "playertarget");
    
    if (not threatpct) then
        threatpct = 0;
    end

    if (threatpct > 100) then
        threatpct = 100;
    end
    
    if (InCombatLockdown()) then
      PlayerFrameAggroBar.background:Show();
    else
      PlayerFrameAggroBar.background:Hide();
    end

    PlayerFrameAggroBar:SetValue(threatpct);
end

function CleanUI_CompactRaidFrameManager_Expand(self)
    self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", -7, -175);
end

function CleanUI_CompactRaidFrameManager_Collapse(self)
    self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", -182, -175);
end

function CleanUI_TargetFrame_CheckFaction(self)
    local name = _G[self:GetName().."TextureFrameName"];
    
    local unitName = GetUnitName(self.unit);
    
    -- change alpha, when tapped
    if ( not UnitPlayerControlled(self.unit) and UnitIsTapDenied(self.unit) ) then
        name:SetTextColor(1.0, 1.0, 1.0, 1.0);
        self:SetAlpha(0.4);
        return;
    end
    
    self:SetAlpha(1.0);
    
    -- background color == class color
    self.nameBackground:SetVertexColor(0.5, 0.5, 0.5);
    
    local localizedClass, englishClass = UnitClass(self.unit);
    local classColor = RAID_CLASS_COLORS[englishClass];

    if (not classColor) then
        classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
    end
        
    self.nameBackground:SetVertexColor(classColor.r, classColor.g, classColor.b);
    
    self.textureFrame.texture:SetDesaturated(1);
    self.textureFrame.texture:SetVertexColor(classColor.r, classColor.g, classColor.b);
    
    -- change text color
    if (UnitPlayerControlled(self.unit) and not UnitIsPVP(self.unit)) then
        name:SetTextColor(1.0, 1.0, 1.0, 1.0);
    elseif (UnitPlayerControlled(self.unit) and UnitIsPVP(self.unit) and UnitFactionGroup("player") == UnitFactionGroup(self.unit)) then
        -- same faction
        name:SetTextColor(1.0, 1.0, 1.0, 1.0);
    elseif (UnitPlayerControlled(self.unit) and UnitIsPVP(self.unit) and UnitFactionGroup("player") ~= UnitFactionGroup(self.unit)) then
        -- other faction
        local englishFaction, localizedFaction = UnitFactionGroup(self.unit);

        if (englishFaction == "Horde") then
            unitName = "|TInterface/WorldStateFrame/HordeIcon:20|t"..unitName;
        elseif (englishFaction == "Alliance") then
            unitName = "|TInterface/WorldStateFrame/AllianceIcon:20|t"..unitName;
        end
        
        local red, green, blue, alpha = UnitSelectionColor(self.unit);
        name:SetTextColor(red, green, blue, alpha);
    else
        local red, green, blue, alpha = UnitSelectionColor(self.unit);
        name:SetTextColor(red, green, blue, alpha);
    end
        
    name:SetText(unitName);
end

function CleanUI_SetUnitFrameDragon(id)
    if (id >= 0 and id <= 3) then
        CleanUIUserData.dragon = id;
        CleanUI_UpdateUnitFrameDragon();
    else
        DEFAULT_CHAT_FRAME:AddMessage("dragon: 0, 1, 2 or 3", 1.0, 0, 0, 1, 10);
    end
end

function CleanUI_SetUnitFrameColoredDragon(coloredDragon)
    CleanUIUserData.coloredDragon = coloredDragon;
end

function CleanUI_UpdateUnitFrameDragon()

    PlayerFrameTexture:SetDesaturated(nil);
    PlayerFrameTexture:SetVertexColor(1.0, 1.0, 1.0);
        
    if (CleanUIUserData.dragon == 1) then
        PlayerFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Elite");
    elseif (CleanUIUserData.dragon == 2) then
        PlayerFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Rare");
    elseif (CleanUIUserData.dragon == 3) then
        PlayerFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Rare-Elite");
    else
        PlayerFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame");
    end

    local localizedClass, englishClass = UnitClass("player");
    local classColor = RAID_CLASS_COLORS[englishClass];

    if (not classColor) then
        classColor = { r = 1.0, g = 1.0, b = 1.0, colorStr = "ffffffff" };
    end

    local desaturated = nil;
    local vertexColor = {r = 1, g = 1, b = 1};

    if (CleanUIUserData.coloredDragon) then
        desaturated = 1;
        vertexColor = classColor;
    end
    
    PlayerFrameTexture:SetDesaturated(desaturated);
    PlayerFrameTexture:SetVertexColor(vertexColor.r, vertexColor.g, vertexColor.b);
    
    if (PlayerFrameAlternateManaBar.cuiborder) then
        PlayerFrameAlternateManaBar.cuiborder:SetDesaturated(desaturated);
        PlayerFrameAlternateManaBar.cuiborder:SetVertexColor(vertexColor.r, vertexColor.g, vertexColor.b);
    end
    
    if (ComboPointPlayerFrame) then
        ComboPointPlayerFrame.Background:SetDesaturated(desaturated);
        ComboPointPlayerFrame.Background:SetVertexColor(vertexColor.r, vertexColor.g, vertexColor.b);
    end
    
    if (PaladinPowerBarFrame) then
        PaladinPowerBarFrameBankBG:SetDesaturated(desaturated);
        PaladinPowerBarFrameBankBG:SetVertexColor(vertexColor.r, vertexColor.g, vertexColor.b);
        PaladinPowerBarFrameBG:SetDesaturated(desaturated);
        PaladinPowerBarFrameBG:SetVertexColor(vertexColor.r, vertexColor.g, vertexColor.b);
    end
    
    if (WarlockPowerFrame) then
        CleanUI_ReplaceAllTextures("UI-WarlockShard", WarlockPowerFrame, "Interface\\AddOns\\CleanUI\\skins\\frame\\CUI-WarlockShard");
        --CleanUI_ReplaceAllTextures("UI-WarlockShard", WarlockPowerFrame, nil);
    end
end

function CleanUI_SetUnitFramesScale(scale)
    if (scale and scale >= 0.75 and scale <= 2) then
        CleanUIUserData.scaleUnitFrames = scale;
        CleanUI_ScaleUnitFrames() 
    else
        DEFAULT_CHAT_FRAME:AddMessage("scale: from 0.75 to 2.0", 1.0, 0, 0, 1, 10);
    end
end

function CleanUI_ScaleUnitFrames()    
    PlayerFrame:SetScale(CleanUIUserData.scaleUnitFrames);
    TargetFrame:SetScale(CleanUIUserData.scaleUnitFrames);
    FocusFrame:SetScale(CleanUIUserData.scaleUnitFrames);
end

function CleanUI_SetPartyScale(scale)
    if (scale and scale >= 0.75 and scale <= 2) then
        CleanUIUserData.scalePartyFrames = scale;
        CleanUI_ScalePartyFrames();
    else
        DEFAULT_CHAT_FRAME:AddMessage("scale: from 0.75 to 2.0", 1.0, 0, 0, 1, 10);
    end
end

function CleanUI_ScalePartyFrames()
    local frame;
    for i=1, 5 do
        frame = _G["PartyMemberFrame"..i];
        
        if (frame) then
            frame:SetScale(CleanUIUserData.scalePartyFrames);
        end
    end
end